package businessLayer.Planilhas;

import java.io.IOException;
import java.util.Date;

import businessLayer.ManipulacaoCliente;

import dataLayer.Cliente;
import dataLayer.Endereco;
import dataLayer.Iterator;
import dataLayer.Excel.ExcelCell;
import dataLayer.Excel.ExcelRow;
import dataLayer.Excel.ExcelSheet;
import dataLayer.Excel.ExcelWorkbook;
import dataLayer.Excel.Exception.CellNumberFormatException;
import dataLayer.Excel.Exception.CellStringFormatException;
import dataLayer.Excel.Exception.IOExcelException;
import dataLayer.Excel.Exception.NotDefinedCellException;
import dataLayer.Excel.Exception.NotDefinedRowException;
import dataLayer.Excel.Exception.SheetNotFoundException;
import dataLayer.Repositorios.Interface.RepositorioCliente;

public class PlanilhaClienteArmazenamento {
	private RepositorioCliente repor;
	
	public PlanilhaClienteArmazenamento(RepositorioCliente recebido)
	{
		repor = recebido;
	}
	
	@SuppressWarnings("deprecation")
	public void colocarClienteExcelNoRepositorio() throws IOExcelException, SheetNotFoundException, NotDefinedRowException, NotDefinedCellException, CellStringFormatException, CellNumberFormatException, IOException 
	{
		ExcelWorkbook workbook = new ExcelWorkbook("Locadora.xls"); //puxa o arquivo xls

		ExcelSheet sheet = workbook.getSheet("Clientes"); // escolhe a planilha do arquivo sobre qual quer trabalhar

		for(int i = 1; i<32; i++)
		{

			ExcelRow row = sheet.getRow(i); // Escolhe a linha que vai fazer a analise
			ExcelCell cell = row.getCell(0); //pega a celula 0 que  referente ao titulo
			String nome = cell.getStringCellValue();  // converte o conteudo da celula em uma string
			
			ExcelCell cell1 = row.getCell(1); // celula 1 referente ao identificador 
			String cpf = cell1.getStringCellValue();  // converte o conteudo da celula em uma string
			if(i != 1) // comea a checar as especificaes da questao, neste caso, checa se o cpf  igual para clientes iguais (o que no pode). O i  diferente de 1 pois  o primeiro cliente e no tem anterior pra ser checado
			{
				for(int j = 1; j<i; j++) // j comea 1 para comparar com os clientes anteriores
				{
					ExcelRow row2 = sheet.getRow(j); 
					ExcelCell cell2 = row2.getCell(1);
					String comparador = cell2.getStringCellValue();
					if(cpf.equals(comparador)) // compara o cpf do cliente em questao com os anteriores pra checar se h cpf igual
					{
						cpf = null; // caso tenha, iguala a null para nao poder ser armazenado no array l embaixo.
					}
				}
			}
			
			ExcelCell cell2 = row.getCell(2); // checa a celula referente ao tipo da copia
			String dataNascimento = cell2.getStringCellValue();
			
			Date nascimento = null;
			
			if(dataNascimento.matches("\\(\\d{2}\\/\\d{2}\\/\\d{4}\\)"))
			{
				int dia = Integer.parseInt(dataNascimento.substring(1, 3));
				int mes = Integer.parseInt(dataNascimento.substring(4, 6));
				int ano = Integer.parseInt(dataNascimento.substring(7, 11));
				nascimento = new Date(ano, mes, dia);
			}
			
			else
			{
				int dia = Integer.parseInt(dataNascimento.substring(1, 3));
				int mes = Integer.parseInt(dataNascimento.substring(4, 5));
				int ano = Integer.parseInt(dataNascimento.substring(6, 10));
				nascimento = new Date(ano, mes, dia);
			}

			ExcelCell cell4 = row.getCell(4);
			String rua = cell4.getStringCellValue();
			
			ExcelCell cell5 = row.getCell(5);
			int numero = (int) cell5.getNumericCellValue();
			
			ExcelCell cell6 = row.getCell(6);
			String complemento = cell6.getStringCellValue();
			
			ExcelCell cell7 = row.getCell(7);
			String bairro = cell7.getStringCellValue();
			
			ExcelCell cell8 = row.getCell(8);
			String cidade = cell8.getStringCellValue();
			// if cidade == null  criar elemento
			
			ExcelCell cell9 = row.getCell(9);
			String estado = cell9.getStringCellValue();
			
			ExcelCell cell11 = row.getCell(11);
			String telefoneFixo = cell11.getStringCellValue();
			
			ExcelCell cell12 = row.getCell(12);
			String telefoneCelular = cell12.getStringCellValue();
			
			//if 11 e 12 == null  criar elemento
			
			ExcelCell cell14 = row.getCell(14);
			String observacao = cell14.getStringCellValue();
			
			if(cidade != "") // if que no armazena no array os filmes que nao atendem as especificaes
			{
				Endereco endereco = new Endereco(rua, complemento, bairro, cidade, estado, numero);
				if(cpf != null && (telefoneFixo != "" || telefoneCelular != ""))
				{
					Cliente suplicante = new Cliente(cpf, nome, nascimento, endereco, telefoneFixo, telefoneCelular, observacao);
					
					Iterator<Cliente> comparador = repor.getIterator();
					
					boolean passa = true;
					
					while(comparador.hasNext())
					{
						if(((Cliente) comparador.next()).getCpf().equals(suplicante.getCpf()))
						{
							passa = false;
						}
					}
					
					if(passa)
					{
						repor.inserirCliente(suplicante);
					}
				}
			}
		}
	}

	public void transferirListaClientes(ManipulacaoCliente parametro) throws IOException
	{
		Iterator<Cliente> novo = repor.getIterator();
		while(novo.hasNext())
		{
			Cliente posicao =  (Cliente) novo.next();
			Endereco enderecoPosicao = ((Cliente) novo.next()).getEndereco();
			parametro.cadastrarCliente(posicao.getCpf(), posicao.getNome(), posicao.getNascimento(), enderecoPosicao.getLogradouro(), enderecoPosicao.getComplemento(), enderecoPosicao.getBairro(), enderecoPosicao.getCidade(), enderecoPosicao.getEstado(), enderecoPosicao.getNumero(), posicao.getTelefoneFix(), posicao.getTelefoneCel(), posicao.getObservacao());
		}
	}
}
