package businessLayer.Planilhas;

import java.io.IOException;
import java.util.Date;

import businessLayer.ManipulacaoLocacao;

import dataLayer.Cliente;
import dataLayer.Copia;
import dataLayer.Endereco;
import dataLayer.Iterator;
import dataLayer.Locacao;
import dataLayer.LocacaoIdentidade;
import dataLayer.Excel.ExcelCell;
import dataLayer.Excel.ExcelRow;
import dataLayer.Excel.ExcelSheet;
import dataLayer.Excel.ExcelWorkbook;
import dataLayer.Excel.Exception.CellNumberFormatException;
import dataLayer.Excel.Exception.CellStringFormatException;
import dataLayer.Excel.Exception.IOExcelException;
import dataLayer.Excel.Exception.NotDefinedCellException;
import dataLayer.Excel.Exception.NotDefinedRowException;
import dataLayer.Excel.Exception.SheetNotFoundException;
import dataLayer.Repositorios.Interface.RepositorioLocacao;

public class PlanilhaLocacaoArmazenado {
	private RepositorioLocacao repor;
	
	public PlanilhaLocacaoArmazenado(RepositorioLocacao recebido)
	{
		repor = recebido;
	} 
	
	private static boolean isBissexto(int ano)
	{
		boolean resposta = false;
		
		if (ano>0 && ano%4 == 0 && (ano%100 == 0 && ano%400 == 0 || ano%100 != 0))
		{
			resposta = true;
		}
		
		return resposta;
	}
	
	@SuppressWarnings("deprecation")
	private static int calcularIntervalo(Date inicioLocacao, Date fimLocacao)
	{
		int intervalo = 0;
		
		if(inicioLocacao.before(fimLocacao))
		{
			if(inicioLocacao.getYear() < fimLocacao.getYear())
			{
				intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 32;
			}
			else
			{
				int mes = inicioLocacao.getMonth();
				if(inicioLocacao.getMonth() == fimLocacao.getMonth())
				{
					intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 1;
				}
				else
				{
					if(mes == 0 || mes == 2 || mes == 4 || mes == 6 || mes == 7 || mes == 9 || mes == 11)
					{
						intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 31;
					}
					else if(mes == 3 || mes == 5 || mes == 8 || mes == 10)
					{
						intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 30;
					}
					else
					{
						if(isBissexto(inicioLocacao.getYear()))
						{
							intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 29;
						}
						else
						{
							intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 28;
						}
					}
				}
			}
		}
		
		return intervalo;
	}
	
	private static boolean compararCpf(String cpfPontos, String cpfContinuo)
	{
		boolean resposta = false;
		String aux = cpfPontos.substring(0,3) + cpfPontos.substring(4,7) + cpfPontos.substring(8,11) + cpfPontos.substring(12,14);
		//System.out.println(cpfPontos + " , " + cpfContinuo);
		if(cpfContinuo.equals(aux))
		{
			resposta = true;
		}
		return resposta;
	}
	
	@SuppressWarnings("deprecation")
	private static Date transformarStringEmDate(String str)
	{
		int dia = Integer.parseInt(str.substring(1, 3));
		int mes = Integer.parseInt(str.substring(4, 6));
		int ano = Integer.parseInt(str.substring(7, 11));
		Date resposta = new Date(ano, mes, dia);
		return resposta;
	}
	
	@SuppressWarnings("deprecation")
	private static Cliente criarCliente(String cpfEntrada, ExcelSheet sheet) throws NotDefinedRowException, NotDefinedCellException, CellStringFormatException, CellNumberFormatException
	{
		Cliente suplicante = null;
		
		for(int i = 1; i<32; i++)
		{
			
			ExcelRow row = sheet.getRow(i); // Escolhe a linha que vai fazer a analise
			ExcelCell cell = row.getCell(1); //pega a celula 0 que  referente ao titulo
			String cpfLocal = cell.getStringCellValue();  // converte o conteudo da celula em uma string
			
			if(compararCpf(cpfLocal, cpfEntrada))
			{
				ExcelCell cell0 = row.getCell(0); // celula 1 referente ao identificador 
				String nome = cell0.getStringCellValue();  // converte o conteudo da celula em uma string
				
				ExcelCell cell1 = row.getCell(1); // celula 1 referente ao identificador 
				String cpf = cell1.getStringCellValue();  // converte o conteudo da celula em uma string
				if(i != 1) // comea a checar as especificaes da questao, neste caso, checa se o cpf  igual para clientes iguais (o que no pode). O i  diferente de 1 pois  o primeiro cliente e no tem anterior pra ser checado
				{
					for(int j = 1; j<i; j++) // j comea 1 para comparar com os clientes anteriores
					{
						ExcelRow row2 = sheet.getRow(j); 
						ExcelCell cell2 = row2.getCell(1);
						String comparador = cell2.getStringCellValue();
						if(cpf.equals(comparador)) // compara o cpf do cliente em questao com os anteriores pra checar se h cpf igual
						{
							cpf = null; // caso tenha, iguala a null para nao poder ser armazenado no array l embaixo.
						}
					}
				}
				
				ExcelCell cell2 = row.getCell(2); // checa a celula referente ao tipo da copia
				String dataNascimento = cell2.getStringCellValue();
				
				Date nascimento = null;
				
				if(dataNascimento.matches("\\(\\d{2}\\/\\d{2}\\/\\d{4}\\)"))
				{
					int dia = Integer.parseInt(dataNascimento.substring(1, 3));
					int mes = Integer.parseInt(dataNascimento.substring(4, 6));
					int ano = Integer.parseInt(dataNascimento.substring(7, 11));
					nascimento = new Date(ano, mes, dia);
				}
				
				else
				{
					int dia = Integer.parseInt(dataNascimento.substring(1, 3));
					int mes = Integer.parseInt(dataNascimento.substring(4, 5));
					int ano = Integer.parseInt(dataNascimento.substring(6, 10));
					nascimento = new Date(ano, mes, dia);
				}
				

				ExcelCell cell4 = row.getCell(4);
				String rua = cell4.getStringCellValue();
				
				ExcelCell cell5 = row.getCell(5);
				int numero = (int) cell5.getNumericCellValue();
				
				ExcelCell cell6 = row.getCell(6);
				String complemento = cell6.getStringCellValue();
				
				ExcelCell cell7 = row.getCell(7);
				String bairro = cell7.getStringCellValue();
				
				ExcelCell cell8 = row.getCell(8);
				String cidade = cell8.getStringCellValue();
				// if cidade == null  criar elemento
				
				ExcelCell cell9 = row.getCell(9);
				String estado = cell9.getStringCellValue();
				
				
				ExcelCell cell11 = row.getCell(11);
				String telefoneFixo = cell11.getStringCellValue();
				
				ExcelCell cell12 = row.getCell(12);
				String telefoneCelular = cell12.getStringCellValue();
				
				//if 11 e 12 == null  criar elemento
				
				ExcelCell cell14 = row.getCell(14);
				String observacao = cell14.getStringCellValue();
				
				if(cidade != "") // if que no armazena no array os filmes que nao atendem as especificaes
				{
					Endereco endereco = new Endereco(rua, complemento, bairro, cidade, estado, numero);
					if(cpf != null && (telefoneFixo != "" || telefoneCelular != ""))
					{
						suplicante = new Cliente(cpf, nome, nascimento, endereco, telefoneFixo, telefoneCelular, observacao);
					}
				}
			}
		}
		return suplicante;
	}

	private static Copia criarCopia(double identificadorEntrada, ExcelSheet sheet) throws CellStringFormatException, NotDefinedCellException, NotDefinedRowException, CellNumberFormatException
	{
		Copia copia = null;
		
		for(int i = 1; i<47; i++)
		{
			ExcelRow row = sheet.getRow(i); // Escolhe a linha que vai fazer a analise
			ExcelCell cell = row.getCell(1); //pega a celula 0 que  referente ao titulo
			double identificadorInterno = cell.getNumericCellValue();  // converte o conteudo da celula em uma string
	
			if(identificadorEntrada == identificadorInterno)
			{
				ExcelCell cell0 = row.getCell(0); //pega a celula 0 que  referente ao titulo
				String titulo = cell0.getStringCellValue();  // converte o conteudo da celula em uma string
				
				ExcelCell cell1 = row.getCell(1); // celula 1 referente ao identificador 
				double identificador = cell1.getNumericCellValue(); // passa o identificador como double para a variavel
				if(i != 1) // comea a checar as especificaes da questao, neste caso, checa se o identificador  igual para titulos iguais (o que no pode). O i  diferente de 1 pois  o primeiro titulo e no tem anterior pra ser checado
				{
					for(int j = 1; j<i; j++) // j comea 1 para comparar com os titulos anteriores
					{
						ExcelRow row2 = sheet.getRow(j); 
						ExcelCell cell2 = row2.getCell(1);
						double comparador = cell2.getNumericCellValue();
						if(identificador == comparador) // compara o identificador do titulo em questao com os anteriores pra checar se h codigo identificador igual
						{
							identificador = 0; // caso tenha, iguala a 0 para nao poder ser armazenado no array l embaixo.
						}
					}
				}
		
				ExcelCell cell2 = row.getCell(2); // checa a celula referente ao tipo da copia
				String categoria = cell2.getStringCellValue();
				if(i!= 1) // faz a checagem tambem, so que agora para os tipos, onde um filme nao pode ter dois tipos diferentes. Essa checagem funciona de forma semelhante a checagem passada.
				{
					ExcelRow rowAuxiliar = sheet.getRow(i-1);
					ExcelCell cellAuxiliar = rowAuxiliar.getCell(0);
					String comparar = cellAuxiliar.getStringCellValue();
					if(comparar.equals(titulo))
					{
						ExcelCell cellAuxiliar2 = rowAuxiliar.getCell(2);
						String categoria2 = cellAuxiliar2.getStringCellValue();
						if(!categoria.equals(categoria2))
						{
							categoria = null; // caso tenham tipos diferentes um mesmo titulo fica igual a null
						}
					}
				}
		
				ExcelCell cell3 = row.getCell(3);
				String tipo = cell3.getStringCellValue();
		
				if(categoria != null && identificador !=0) // if que no armazena no array os filmes que nao atendem as especificaes
				{
					copia = new Copia(titulo, identificador, categoria, tipo);
				}
			}
			
		}
		
		return copia;
	}
	
	public void colocarLocacaoExcelNoRepositorio() throws IOExcelException, SheetNotFoundException, NotDefinedRowException, NotDefinedCellException, CellStringFormatException, CellNumberFormatException, IOException 
	{
		ExcelWorkbook workbook = new ExcelWorkbook("Locadora.xls"); //puxa o arquivo xls

		ExcelSheet sheet = workbook.getSheet("Locacoes"); // escolhe a planilha do arquivo sobre qual quer trabalhar
		ExcelSheet sheetAux = workbook.getSheet("Filmes"); // escolhe a planilha do arquivo sobre qual quer trabalhar
		ExcelSheet sheetOutro = workbook.getSheet("Clientes"); // escolhe a planilha do arquivo sobre qual quer trabalhar

		for(int i = 1; i<19; i++)
		{
			
			boolean achouCopia = false;
			boolean achouCliente = false;
			boolean achouDataInvalida = false;
			
			ExcelRow row = sheet.getRow(i); // Escolhe a linha que vai fazer a analise
			ExcelCell cell = row.getCell(0); //pega a celula 0 que  referente ao titulo
			double identificadorCopia = cell.getNumericCellValue();  // converte o conteudo da celula em uma string
			
			for(int j = 1; j<47; j++)
			{
				ExcelRow rowAux = sheetAux.getRow(j); // Escolhe a linha que vai fazer a analise
				ExcelCell cellAux = rowAux.getCell(1); //pega a celula 0 que  referente ao titulo
				double identificadorCopiaAux = cellAux.getNumericCellValue();  // converte o conteudo da celula em uma string
				if(identificadorCopia == (identificadorCopiaAux))
				{
					achouCopia = true;
				}
			}
			
			if(!achouCopia)
			{
				identificadorCopia = 0;
			}
			
			String aux = null;
	
			ExcelCell cell1 = row.getCell(1); //pega a celula 0 que  referente ao titulo
			double cpfCliente = cell1.getNumericCellValue();
			String stringGenerator = cpfCliente + "";
			if(stringGenerator.substring(11,12).equals("E"))
			{
				aux = stringGenerator.substring(0, 1) + stringGenerator.substring(2,11)+ "0";
			}
			else
			{
				aux = stringGenerator.substring(0, 1) + stringGenerator.substring(2,12);
			}

			
			for(int j = 1; j<32; j++)
			{
				ExcelRow rowAux = sheetOutro.getRow(j); // Escolhe a linha que vai fazer a analise
				ExcelCell cellAux = rowAux.getCell(1); //pega a celula 0 que  referente ao titulo
				String cpfClienteAux = cellAux.getStringCellValue();  // converte o conteudo da celula em uma string
				if(compararCpf(cpfClienteAux, aux))
				{
					achouCliente = true;
				}
			}
			
			if(!achouCliente)
			{
				aux = null;
			}
			
			ExcelCell cell2 = row.getCell(2); //pega a celula 0 que  referente ao titulo
			String dataInicio = cell2.getStringCellValue();  // converte o conteudo da celula em uma string
			
			if(i != 1) // comea a checar as especificaes da questao, neste caso, checa se o identificador  igual para titulos iguais (o que no pode). O i  diferente de 1 pois  o primeiro titulo e no tem anterior pra ser checado
			{
				for(int j = 1; j<i; j++) // j comea 1 para comparar com os titulos anteriores
				{
					ExcelRow row2 = sheet.getRow(j); 
					ExcelCell cell2Aux = row2.getCell(0);
					
					double comparadorCopia = cell2Aux.getNumericCellValue();
					
					if(comparadorCopia == identificadorCopia)
					{
						ExcelCell cell2AuxData = row2.getCell(3);
						String comparadorData = cell2AuxData.getStringCellValue();
						
						if(transformarStringEmDate(dataInicio).before(transformarStringEmDate(comparadorData)))
						{
							achouDataInvalida = true;
						}
					}	
				}
			}
			
			ExcelCell cell3 = row.getCell(3); //pega a celula 0 que  referente ao titulo
			String dataFinal = cell3.getStringCellValue();  // converte o conteudo da celula em uma string
			
			if((calcularIntervalo(transformarStringEmDate(dataInicio), transformarStringEmDate(dataFinal))) > 6)
			{
				dataInicio = null;
			}
			
			if(achouDataInvalida)
			{
				dataInicio = null;
			}
			
			if(identificadorCopia != 0 && aux != null && dataInicio != null)
			{
				Cliente cliente = criarCliente(aux, sheetOutro);
				
				if(cliente != null)
				{
					Copia copia = criarCopia(identificadorCopia, sheetAux);
					
					if(copia != null)
					{	
						int id = 1;
						LocacaoIdentidade locacao = new LocacaoIdentidade(cliente, copia, transformarStringEmDate(dataInicio), transformarStringEmDate(dataFinal), id);
						Iterator<LocacaoIdentidade> comparador = repor.getIterator();
						
						boolean passa = true;
						
						while(comparador.hasNext())
						{
							LocacaoIdentidade posicao = (LocacaoIdentidade) comparador.next();
							if(posicao.getCliente().getCpf().equals(locacao.getCliente().getCpf()))
							{
								if(posicao.getCopia().getCodigo() == (locacao.getCopia().getCodigo()))
								{
									if(posicao.getInicioLocacao().equals(locacao.getInicioLocacao()) && posicao.getFimLocacao().equals(locacao.getFimLocacao()))
									{
										passa = false;
									}
								}
							}
						}
						
						if(passa)
						{
							repor.fazerLocacao(locacao);
						}
					}
				}
			}
		}
	}

	public void trasnferirListaLocacoes(ManipulacaoLocacao parametro) throws IOException
	{
		System.out.println(">>>>LoopComeo<<<<");
		Iterator<LocacaoIdentidade> novo = repor.getIterator();
		while(novo.hasNext())
		{
			System.out.println(">>>>LoopDentro<<<<");
			LocacaoIdentidade posicao = (LocacaoIdentidade) novo.next();
			System.out.println("1");
			parametro.locarFilme(posicao.getCliente(), posicao.getCopia(), posicao.getInicioLocacao(), posicao.getFimLocacao());
			System.out.println("2");
		}
		System.out.println(">>>>LoopFim<<<<");
	}
}
