package dataLayer;

import java.io.Serializable;
import java.util.Date;

public class Locacao implements Serializable{
	
	private static boolean isBissexto(int ano)
	{
		boolean resposta = false;
		
		if (ano>0 && ano%4 == 0 && (ano%100 == 0 && ano%400 == 0 || ano%100 != 0))
		{
			resposta = true;
		}
		
		return resposta;
	}
	
	// Identidade nica ainda no marcada.
	
	/*A planliha Locaes do arquivo Locadora.xls contem algumas locaes, relacionando o CPF dos clientes com o cdigo identificador das cpias. Os campos marcados com asteriscos so obrigatrios e o campo valor  calculado automaticamente, de acordo com as regras abaixo.

	O perodo de locao pode variar entre 1 e 6 dias. O primeiro dia de locao sempre custa o valor indicado na cpia. Para cada dia excedente, o valor restante tem o desconto aumentado em 10%. Por exemplo, o aluguel de uma cpia que custa R$ 4,00 por 5 dias custa: R$ 4,00 + 4 * R$ 4,00 * (100% - 40%) = R$ 13,60.

	A cada 3 locaes de cpias do tipo catlogo, o cliente ganha um outro catlogo de graa (valor da locao R$ 0,00). A cada 6 locaes de cpias do tipo lanamento, o cliente tambm ganha outro lanamento de graa. Uma cpia pode ser locada apenas se estiver disponvel no perodo. */ 

	
	private static final long serialVersionUID = 42L;
		private Cliente cliente;
		private Copia copia;
		private Date inicioLocacao;
		private Date fimLocacao;
		private double valor;
		private int intervalo;
		
		@SuppressWarnings("deprecation")
		public Locacao(Cliente cliente, Copia copia, Date inicioLocacao, Date fimLocacao) {
			this.cliente = cliente;
			this.copia = copia;
			this.inicioLocacao = inicioLocacao;
			this.fimLocacao = fimLocacao;
			
			if(inicioLocacao.before(fimLocacao))
			{
				if(inicioLocacao.getYear() < fimLocacao.getYear())
				{
					intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 32;
				}
				else
				{
					int mes = inicioLocacao.getMonth();
					if(inicioLocacao.getMonth() == fimLocacao.getMonth())
					{
						intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 1;
					}
					else
					{
						if(mes == 0 || mes == 2 || mes == 4 || mes == 6 || mes == 7 || mes == 9 || mes == 11)
						{
							intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 31;
						}
						else if(mes == 3 || mes == 5 || mes == 8 || mes == 10)
						{
							intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 30;
						}
						else
						{
							if(isBissexto(inicioLocacao.getYear()))
							{
								intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 29;
							}
							else
							{
								intervalo = fimLocacao.getDate() - inicioLocacao.getDate() + 28;
							}
						}
					}
				}
				
				double valorTempo = intervalo * copia.getValor() * (1 - (intervalo/10));
				this.valor = copia.getValor() + valorTempo;
			}
			else
			{
				this.valor = copia.getValor();
			}
		}

		public void setCliente(Cliente cliente) {
			this.cliente = cliente;
		}
		public Cliente getCliente() {
			return cliente;
		}

		public void setCopia(Copia copia) {
			this.copia = copia;
		}
		public Copia getCopia() {
			return copia;
		}

		public void setValor(double valor) {
			this.valor = valor;
		}
		public double getValor() {
			return valor;
		}

		public void setInicioLocacao(Date inicioLocacao) {
			this.inicioLocacao = inicioLocacao;
		}

		public Date getInicioLocacao() {
			return inicioLocacao;
		}

		public void setFimLocacao(Date fimLocacao) {
			this.fimLocacao = fimLocacao;
		}

		public Date getFimLocacao() {
			return fimLocacao;
		}
		
		@SuppressWarnings("deprecation")
		public String toString()
		{
			Date auxInicio = this.getInicioLocacao();
			Date auxFim = this.getFimLocacao();
			
			String inicio = auxInicio.getDate()+"/"+auxInicio.getMonth()+"/"+auxInicio.getYear();
			String fim = auxFim.getDate()+"/"+auxFim.getMonth()+"/"+auxFim.getYear();
			
			String dados = this.getCliente().getNome() + " | " + this.getCopia().getCodigo() + " | " + this.getCopia().getNome()
			+ " | " + inicio + " - " + fim + " | " + intervalo + " dias | R$ " + this.getValor();
			
			return dados;
		}
		
		@SuppressWarnings("deprecation")
		public String printString()
		{
			Date auxInicio = this.getInicioLocacao();
			Date auxFim = this.getFimLocacao();
			
			String inicio = auxInicio.getDate()+"/"+auxInicio.getMonth()+"/"+auxInicio.getYear();
			String fim = auxFim.getDate()+"/"+auxFim.getMonth()+"/"+auxFim.getYear();
			
			String dadosCopia = this.getCopia().toString();
			String dadosCliente = this.getCliente().toString();
			String resposta = dadosCliente + "\n" + dadosCopia + "\n" + inicio + " - " + fim;
			
			return resposta;
		}
		
	}
